var mouseCX = 0; // Store the position of the mouse coordinate in X.
var prevMouseCX = 0; // Store the previous mouse coordinate in X.

// HTML Elements
var mainmenu = document.getElementById("mainmenu");
mainmenu.classList.remove("header_menu_inner");
var content  = document.getElementById("content-inner");
var cbMenu = document.getElementById("cbMenu");

var mouseDown = false;
var distanceCX = 0;

window.onresize = (() => {
    if(window.innerWidth > 1200 && cbMenu.checked)
    {
        cbMenu.checked = false;
        var matrixValues = menu_getTransformMatrix();
        menu_slideLeft(matrixValues[4]);
    }
});

// If the menu button is toggled, open the menu. Else, close the menu.
cbMenu.onchange = function(event) {
    var matrixValues = menu_getTransformMatrix();
    if(cbMenu.checked)
    {
        menu_slideRight(matrixValues[4]);
        mainmenu.animate([
            { boxShadow: '0 0em 0em 0em rgba(0,0,0,0.5)' },
            { boxShadow: '0 2em 2em 2em rgba(0,0,0,0.5)' }],
            { duration: 500 });
        mainmenu.style.boxShadow = '0 2em 2em 2em rgba(0,0,0,0.5)';
    }
    else
    {
        menu_slideLeft(matrixValues[4]);
        mainmenu.animate([
            { boxShadow: '0 2em 2em 2em rgba(0,0,0,0.5)' },
            { boxShadow: '0 0em 0em 0em rgba(0,0,0,0.5)' }],
            { duration: 500 });
        mainmenu.style.boxShadow = '0 0em 0em 0em rgba(0,0,0,0.5)';
    }
}

// Get the current x-position of the mouse and store in mouseCX. "C" is "client."
// This allows the user to slide the side menu in and out.
var menuInMotion = function(event) {
    // If using a touch screen, this is a one-finger operation. Otherwise, the
    // user is using a mouse.
    mouseCX = (typeof event.touches !== "undefined") ? event.touches[0].clientX :
        event.clientX;
    if(mouseDown) {
        // Math.min(Math.max(min, val), max)
        // Increase or decrease the distance between the left edge of the screen
        // and where the mouse cursor is located.
        distanceCX += mouseCX - prevMouseCX;
        mainmenu.style.transform = 'translateX('
            + Math.min(Math.max(0, distanceCX), mainmenu.offsetWidth) + 'px)';
    }
    // Store the previous position of the mouse cursor. This will be used to
    // calculate the direction the mouse cursor is going.
    prevMouseCX = mouseCX;
    
    if(cbMenu.checked || mouseCX < 200)
        return false;
}

//document.body.onmousemove = menuInMotion;
//document.body.ontouchmove = menuInMotion;

// User wants to slide the menu out from the side or slide the menu
// back towards the left. Start moving the menu.
var beginMenuMovement = function()
{
    // If the menu is open, only allow the user to swipe inside the menu bar.
    // Otherwise, if the menu is closed, the user can only slide out from the
    // left edge of the web page.
    // Only move the menu if the width of the viewport is 1200 pixels or less.
    if(window.innerWidth <= 1200
        && ((cbMenu.checked && this.matches('#mainmenu')) || mouseCX < 20))
    {
        mouseDown = true;
        content.style.userSelect = "none";
        if(!cbMenu.checked)
        {
            mainmenu.animate([
                { boxShadow: '0 0em 0em 0em rgba(0,0,0,0.5)' },
                { boxShadow: '0 2em 2em 2em rgba(0,0,0,0.5)' }],
                { duration: 100 });
            mainmenu.style.boxShadow = '0 2em 2em 2em rgba(0,0,0,0.5)';
        }   
    }
}


//mainmenu.onmousedown = beginMenuMovement;
//content.onmousedown = beginMenuMovement;
//mainmenu.ontouchstart = beginMenuMovement;
//content.ontouchstart = beginMenuMovement;

// User has finished moving the side menu bar.
var endMenuMovement = function()
{
    // Same for when the user releases the left mouse button.
    if(window.innerWidth <= 1200)
    {
        mouseDown = false;
        // Get the translation values for the side bar menu.
        var matrixValues = menu_getTransformMatrix();
        // If the user slides the menu as far to the left,
        // the side menu bar is closed (unchecked).
        menu_minWidth = mainmenu.offsetWidth / 2;
        if(matrixValues[4] > menu_minWidth)
        {
            cbMenu.checked = true;
            menu_slideRight(matrixValues[4]);
        }
        else
        {
            cbMenu.checked = false;
            menu_slideLeft(matrixValues[4]);
            var boxShadow = window.getComputedStyle(mainmenu).boxShadow;
            console.log(boxShadow != 'rgba(0, 0, 0, 0.5) 0px 0px 0px 0px');
            if(boxShadow != 'rgba(0, 0, 0, 0.5) 0px 0px 0px 0px')
            {
                mainmenu.animate([
                    { boxShadow: '0 2em 2em 2em rgba(0,0,0,0.5)' },
                    { boxShadow: '0 0em 0em 0em rgba(0,0,0,0.5)' }],
                    { duration: 500 });
                mainmenu.style.boxShadow = '0 0em 0em 0em rgba(0,0,0,0.5)';
            }
        }
        content.style.userSelect = "";
    }
}

//mainmenu.onmouseup = endMenuMovement;
//content.onmouseup = endMenuMovement;
//mainmenu.ontouchend = endMenuMovement;
//content.ontouchend = endMenuMovement;

// The two functions below this comment is for performing animations and for
// holding at the start or end position of the side menu bar's left coordinate.
function menu_slideRight(matrixValues)
{
    // Play the animation that starts from where the user releases the mouse
    // button and slide to the end, which equals the width of the menu.
    // Source: https://developer.mozilla.org/en-US/docs/Web/API/Element/animate
    mainmenu.animate([
        { transform: 'translateX(' + matrixValues + 'px)' },
        { transform: 'translateX(' + mainmenu.offsetWidth + 'px)' }],
        { duration: 500 });
    // Set the new transform position to the width of the menu.
    mainmenu.style.transform = 'translateX(' + mainmenu.offsetWidth + 'px)';
    distanceCX = mainmenu.offsetWidth;
}

function menu_slideLeft(matrixValues)
{
    mainmenu.animate([
        { transform: 'translateX(' + matrixValues + 'px)' },
        { transform: 'translateX(0px)' }],
        { duration: 500 });
    mainmenu.style.transform = 'translateX(0px)';
    distanceCX = 0;
}

// Source: https://zellwk.com/blog/css-translate-values-in-javascript/
function menu_getTransformMatrix() {
    var matrix = window.getComputedStyle(mainmenu).transform;
    return matrix.slice(7, -1).split(', ');
}
